==> ec-rider0.8.1.tar.gz: ec-rider0.8.1/ec_admin/app/models_admin/cms_version.rb <==
# -*- coding: utf-8 -*-
require 'fileutils'
require 'tmpdir'

class CmsVersion
  attr_accessor :dir, :git, :logger

  def initialize(dir, logger = nil)
    @dir = dir
    @git = Git.open(dir, :log => logger)
  end
  
  def new_commit(attributes = {})
    git_commit = CmsVersion::Commit.new(attributes)
    git_commit.setup(@git)
    git_commit
  end

  def find_commit(commit_id)
    @git.object(commit_id)
  end

  def revert(commit_id, path)
    git_commit = @git.object(commit_id)
    log = @git.log(1000) # 最大1000件
    log.between(commit_id, "HEAD") # params[:id]で指定されるコミットは含まれません。
    log.path(path)
    tmp_backup = File.join(Dir.tmpdir, File.basename(path))
    FileUtils.cd(EcRider.config.cms_git_dir) do
      log.each do |commit|
        execute_command('git revert "-n" "%s"' % commit.sha)
      end
      FileUtils.cp(path, tmp_backup)
      @git.reset_hard("HEAD")
      FileUtils.cp(tmp_backup, path)
      execute_command('git commit "-a" "-m" "%s"' % "#{path}を#{git_commit.date.strftime('%Y/%m/%d %H:%M:%S')}の変更を元に戻しました")
    end
  end

  def reset_hard_with_backup_branch(commit_id)
    @git.branch(Time.now.strftime('%Y%m%d%H%M%S')).create
    @git.reset_hard(commit_id)
  end

  def filtered_log(filter_form)
    filter_form.filter(@git)
  end

  def sorted_branche_names
    return @git.branches.select{|branch| branch.name =~ /^\d{14}$/}.sort{|a, b| b.name <=> a.name}
  end

  def checkout(branch_id)
    # 現在のブランチの名前を変更する
    dir = EcRider.config.cms_git_dir
    system('cd %s && git branch -m %s %s' % [dir, current_branch(@git).name, Time.now.strftime('%Y%m%d%H%M%S')])
    # ブランチを切り替え、ブランチ名をmasterにする
    @git.branch(branch_id).checkout
    system('cd %s && git branch -m %s %s' % [dir, current_branch(@git).name, 'master'])
  end
  
  def delete_branch(branch_id)
    @git.branch(branch_id).delete
  end

  private

  def execute_command(cmd)
    if @logger
      @logger.debug(cmd)
      @logger.debug(`#{cmd}`)
    else
      system(cmd)
    end
  end

  def current_branch(git)
    git.branches.each do |branch|
      return branch if branch.current
    end
  end
  

  public

  class LogSearchForm < ActiveForm
    # gems/git/log.rb の Git::Logの属性名に合わせてあります。
    
    PARAMETER_NAMES = [
      :since, :until, # "2009/10/16 14:30"とか、"2 weeks"とか
      # :count, # 何件取得するか
      :skip,    # 何件スキップするか、オリジン。
      :grep,   # コメントのメッセージに対するフィルタ。文字列
      :path, # リポジトリのルートからのパス、ディレクトリでもOK
      # 以下未実装
      # :author, # コミットしたユーザー
      # :object, # objectish??
      # :between # ['12345678~3', 'HEAD^'] とか
    ]
    attr_accessor(*PARAMETER_NAMES)

    def filter(git)
      self.since = self.since.strftime("%Y-%m-%d") if self.since.is_a?(Time) || self.since.is_a?(Date)
      self.until = self.until.strftime("%Y-%m-%d") if self.until.is_a?(Time) || self.until.is_a?(Date)
      
      log = git.log(count)
      PARAMETER_NAMES.each do |attr|
        value = self.send(attr)
        log.send(attr, value) unless value.blank?
      end
      log
    end

    def page
      (self.skip.to_i / self.count) + 1
    end

    def page=(value)
      self.skip = self.count * ([value.to_i, 1].max - 1)
    end

    def count
      @count || EcRider::ConditionForm::DEFAULT_PER_PAGE
    end
    
    def count=(value)
      value = value.to_i
      @count = (value < 1) ? EcRider::ConditionForm::DEFAULT_PER_PAGE : value
    end
  end
  
  class Commit < ActiveForm
    attr_accessor :current
    attr_accessor :last_log_objectish
    attr_accessor :addeds, :deleteds, :untrackeds
    attr_accessor :message

    def count
      count = 0
      count += addeds.size if addeds
      count += deleteds.size if deleteds
      count += untrackeds.size if untrackeds
    end
    
    def empty?
      count == 0
    end

    def setup(git)
      @git = git
      @current = Commit.new
      @current.load_from_git(@git)
      @last_log_objectish ||= @current.last_log_objectish
      @addeds ||= []
      @deleteds ||= []
      @untrackeds ||= []
      self
    end

    def load_from_git(git)
      last_log = git.log(1)
      @last_log_objectish = last_log.first.objectish
      stat = git.status
      @addeds = filenames(stat.changed) # 変更
      @deleteds = filenames(stat.deleted) # 削除
      @untrackeds = filenames(stat.untracked) # 追加
    end

    def filenames(array)
      array.map{|item|item.first}.sort
    end

    def validate
      return unless current
      if last_log_objectish != current.last_log_objectish
        errors.add_to_base("すでにバージョン管理情報が更新されています")
        return
      end
      %w(addeds deleteds untrackeds).each do |attr|
        my_attr = send(attr)
        current_attr = current.send(attr)
        my_attr.each do |value|
          unless current_attr.include?(value)  
            errors.add_to_base("すでにバージョン管理情報が更新されています")
          end
        end
      end
    end

    def apply
      @git.add(addeds) unless addeds.empty? # 変更
      @git.add(untrackeds) unless untrackeds.empty? # 追加
      @git.remove(deleteds) unless deleteds.empty? # 削除
      @git.commit(message.inspect)
    end
  end
  
end

==> ec-rider0.8.2.tar.gz: ec-rider0.8.2/ec_admin/app/models_admin/cms_version.rb <==
# -*- coding: utf-8 -*-
require 'fileutils'
require 'tmpdir'

class CmsVersion
  attr_accessor :dir, :git, :logger

  def initialize(dir, logger = nil)
    @dir = dir
    @git = Git.open(dir, :log => logger)
  end
  
  def new_commit(attributes = {})
    git_commit = CmsVersion::Commit.new(attributes)
    git_commit.setup(@git)
    git_commit
  end

  def find_commit(commit_id)
    @git.object(commit_id)
  end

  # git revert は、バージョンを戻す新しいコミットを作ります
  def revert(commit_id, path)
    git_commit = @git.object(commit_id)
    log = @git.log(1000) # 最大1000件
    log.between(commit_id, "HEAD") # params[:id]で指定されるコミットは含まれません。
    log.path(path)
    tmp_backup = File.join(RAILS_ROOT, "tmp", File.basename(path))
    FileUtils.cd(EcRider.config.cms_git_dir) do
      log.each do |commit|
        execute_command('git revert "-n" "%s"' % commit.sha)
      end
      FileUtils.cp(path, tmp_backup)
      @git.reset_hard("HEAD")
      FileUtils.cp(tmp_backup, path)
      execute_command('git commit "-a" "-m" "%s"' % "#{path}を#{git_commit.date.strftime('%Y/%m/%d %H:%M:%S')}の変更を元に戻しました")
    end
  end

  # git reset --hard は戻したい時点までのコミットをすべて取り消します
  def reset_hard_with_backup_branch(commit_id)
    @git.branch(Time.now.strftime('%Y%m%d%H%M%S')).create
    @git.reset_hard(commit_id)
  end

  def filtered_log(filter_form)
    filter_form.filter(@git)
  end

  def sorted_branche_names
    # sort{...} よりも sort_by{|branch| branch.name} の方が良いです
    return @git.branches.select{|branch| branch.name =~ /^\d{14}$/}.sort{|a, b| b.name <=> a.name}
  end

  def checkout(branch_id)
    # 現在のブランチの名前を変更する
    dir = EcRider.config.cms_git_dir
    system('cd %s && git branch -m %s %s' % [dir, current_branch(@git).name, Time.now.strftime('%Y%m%d%H%M%S')])
    # ブランチを切り替え、ブランチ名をmasterにする
    @git.branch(branch_id).checkout
    system('cd %s && git branch -m %s %s' % [dir, current_branch(@git).name, 'master'])
  end
  
  def delete_branch(branch_id)
    @git.branch(branch_id).delete
  end

  private

  def execute_command(cmd)
    if @logger
      @logger.debug(cmd)
      @logger.debug(`#{cmd}`)
    else
      system(cmd)
    end
  end

  def current_branch(git)
    # detectを使いましょう
    # git.branches.detect{|b| b.current}
    git.branches.each do |branch|
      return branch if branch.current
    end
  end
  

  public

  class LogSearchForm < ActiveForm
    # gems/git/log.rb の Git::Logの属性名に合わせてあります。
    
    PARAMETER_NAMES = [
      :since, :until, # "2009/10/16 14:30"とか、"2 weeks"とか
      # :count, # 何件取得するか
      :skip,    # 何件スキップするか、オリジン。
      :grep,   # コメントのメッセージに対するフィルタ。文字列
      :path, # リポジトリのルートからのパス、ディレクトリでもOK
      # 以下未実装
      # :author, # コミットしたユーザー
      # :object, # objectish??
      # :between # ['12345678~3', 'HEAD^'] とか
    ]
    attr_accessor(*PARAMETER_NAMES)

    def filter(git)
      self.since = self.since.strftime("%Y-%m-%d") if self.since.is_a?(Time) || self.since.is_a?(Date)
      self.until = self.until.strftime("%Y-%m-%d") if self.until.is_a?(Time) || self.until.is_a?(Date)
      
      log = git.log(count)
      PARAMETER_NAMES.each do |attr|
        value = self.send(attr)
        log.send(attr, value) unless value.blank?
      end
      log
    end

    def page
      (self.skip.to_i / self.count) + 1
    end

    def page=(value)
      self.skip = self.count * ([value.to_i, 1].max - 1)
    end

    def count
      @count || EcRider::ConditionForm::DEFAULT_PER_PAGE
    end
    
    def count=(value)
      value = value.to_i
      @count = (value < 1) ? EcRider::ConditionForm::DEFAULT_PER_PAGE : value
    end
  end
  
  class Commit < ActiveForm
    attr_accessor :current
    attr_accessor :last_log_objectish
    attr_accessor :addeds, :deleteds, :untrackeds
    attr_accessor :message

    def count
      count = 0
      count += addeds.size if addeds
      count += deleteds.size if deleteds
      count += untrackeds.size if untrackeds
    end
    
    def empty?
      count == 0
    end

    def setup(git)
      @git = git
      @current = Commit.new
      @current.load_from_git(@git)
      @last_log_objectish ||= @current.last_log_objectish
      @addeds ||= []
      @deleteds ||= []
      @untrackeds ||= []
      self
    end

    def load_from_git(git)
      last_log = git.log(1)
      @last_log_objectish = last_log.first.objectish
      stat = git.status
      @addeds = filenames(stat.changed) # 変更
      @deleteds = filenames(stat.deleted) # 削除
      @untrackeds = filenames(stat.untracked) # 追加
    end

    def filenames(array)
      array.map{|item|item.first}.sort
    end

    def validate
      return unless current
      if last_log_objectish != current.last_log_objectish
        errors.add_to_base("すでにバージョン管理情報が更新されています")
        return
      end
      %w(addeds deleteds untrackeds).each do |attr|
        my_attr = send(attr)
        current_attr = current.send(attr)
        my_attr.each do |value|
          unless current_attr.include?(value)  
            errors.add_to_base("すでにバージョン管理情報が更新されています")
          end
        end
      end
    end

    def apply
      @git.add(addeds) unless addeds.empty? # 変更
      @git.add(untrackeds) unless untrackeds.empty? # 追加
      @git.remove(deleteds) unless deleteds.empty? # 削除
      @git.commit(message.inspect)
    end
  end
  
end
